@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,3 +26,8 @@ user_agent = 'gPodder'
 # Are we running in GUI, Maemo or console mode?
 interface = CLI
 
+# D-Bus specific interface names
+dbus_bus_name = 'org.godder'
+dbus_gui_object_path = '/gui'
+dbus_interface = 'org.gpodder.interface'
+
