@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -75,6 +75,7 @@ gPodderSettings = {
     'ipod_write_gtkpod_extended': (bool, False),
     'ipod_purge_old_episodes': (bool, False),
     'mp3_player_delete_played': (bool, False),
+    'disable_pre_sync_conversion': (bool, False),
     
     # Tray icon and notification settings
     'display_tray_icon': (bool, False),
@@ -97,7 +98,7 @@ gPodderSettings = {
 
     # Playlist Management settings
     'mp3_player_playlist_file': (str, 'PLAYLISTS/gpodder.m3u'),
-    'mp3_player_playlist_absolute_path': (bool, False),
+    'mp3_player_playlist_absolute_path': (bool, True),
     'mp3_player_playlist_win_path': (bool, True),
 
     # Special settings (not in preferences)
@@ -116,7 +117,6 @@ gPodderSettings = {
     'custom_player_coverart_size' : (int, 176),
     'custom_player_coverart_name' : (str, 'folder.jpg'),
     'custom_player_coverart_format' : (str, 'JPEG'),
-    'experimental_file_naming': (bool, False),
     'podcast_list_icon_size': (int, 32),
     'cmd_all_downloads_complete': (str, ''),
     'cmd_download_complete': (str, ''),
@@ -124,6 +124,10 @@ gPodderSettings = {
     'max_simulaneous_feeds_updating': (int, 3),
     'color_updating_feeds': (str, '#7db023'),
     'log_sqlite': (bool, False),
+    'enable_html_shownotes': (bool, True),
+    'maemo_enable_gestures': (bool, False),
+    'sync_disks_after_transfer': (bool, True),
+    'resume_ask_every_episode': (bool, False),
 
     # Hide the cover/pill from the podcast sidebar when it gets too small
     'podcast_sidebar_save_space': (bool, False),
@@ -178,7 +182,7 @@ class Config(dict):
             ( fieldtype, default ) = self.Settings[name]
             return self[name]
         else:
-            raise AttributeError
+            raise AttributeError('%s is not a setting' % name)
 
     def add_observer(self, callback):
         """
