@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -118,7 +118,7 @@ def sync_device():
         msg('error', _('No device configured. Please use the GUI.'))
         return False
 
-    callback_status = lambda s: msg('status', s)
+    callback_status = lambda s: msg('status', '%s', s)
     device.register('status', callback_status)
     callback_done = lambda: msg('done', _('Synchronization finished.'))
     device.register('done', callback_done)
