@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -72,6 +72,9 @@ class ConverterCollection( types.DictType):
     def has_converter(self, extension):
         if util.find_command('lame') is not None:
             extension = extension.lower()
+            if len(extension) == 0:
+                log('Cannot find a converter without extension.', sender=self)
+                return False
             if extension[0] == '.':
                 extension = extension[1:]
             return self.has_key(extension)
