@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -287,25 +287,27 @@ class gPodderLib(object):
     def format_filesize(self, bytesize, digits=2):
         return util.format_filesize(bytesize, self.config.use_si_units, digits)
 
-    def clean_up_downloads( self, delete_partial = False):
-        # Clean up temporary files left behind by old gPodder versions
-        if delete_partial:
-            temporary_files = glob.glob( '%s/*/.tmp-*' % ( self.downloaddir, ))
-            for tempfile in temporary_files:
-                util.delete_file( tempfile)
+    def find_partial_files(self):
+        return glob.glob(os.path.join(self.downloaddir, '*', '*.partial'))
 
-        # Clean up empty download folders
-        download_dirs = glob.glob( '%s/*' % ( self.downloaddir, ))
+    def clean_up_downloads(self, delete_partial=False):
+        # Clean up temporary files left behind by old gPodder versions
+        temporary_files = glob.glob('%s/*/.tmp-*' % self.downloaddir)
+
+        if delete_partial:
+            temporary_files += glob.glob('%s/*/*.partial' % self.downloaddir)
+
+        for tempfile in temporary_files:
+            util.delete_file(tempfile)
+
+        # Clean up empty download folders and abandoned download folders
+        download_dirs = glob.glob(os.path.join(self.downloaddir, '*'))
         for ddir in download_dirs:
-            if os.path.isdir( ddir):
-                globr = glob.glob( '%s/*' % ( ddir, ))
-                if not globr:
-                    log( 'Stale download directory found: %s', os.path.basename( ddir))
-                    try:
-                        os.rmdir( ddir)
-                        log( 'Successfully removed %s.', ddir)
-                    except:
-                        log( 'Could not remove %s.', ddir)
+            if os.path.isdir(ddir) and not db.channel_foldername_exists(os.path.basename(ddir)):
+                globr = glob.glob(os.path.join(ddir, '*'))
+                if len(globr) == 0 or (len(globr) == 1 and globr[0].endswith('/cover')):
+                    log('Stale download directory found: %s', os.path.basename(ddir), sender=self)
+                    shutil.rmtree(ddir, ignore_errors=True)
 
     def get_download_dir( self):
         util.make_directory( self.config.download_dir)
@@ -350,7 +352,8 @@ class gPodderLib(object):
             db.save_episode(episode)
             filename = episode.url
         else:
-            filename = episode.local_filename()
+            filename = episode.local_filename(create=False)
+            assert filename is not None
         db.mark_episode(episode.url, is_played=True)
 
         if gpodder.interface == gpodder.MAEMO and not self.config.maemo_allow_custom_player:
