@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ import urlparse
 import time
 import datetime
 import rfc822
-import md5
+import hashlib
 import xml.dom.minidom
 import feedparser
 
@@ -81,6 +81,7 @@ class HTTPAuthError(Exception): pass
 class podcastChannel(object):
     """holds data for a complete channel"""
     SETTINGS = ('sync_to_devices', 'device_playlist_name','override_title','username','password')
+    MAX_FOLDERNAME_LENGTH = 150
     icon_cache = {}
 
     fc = cache.Cache()
@@ -125,8 +126,16 @@ class podcastChannel(object):
         if c.status == 401:
             return ( False, 401 )
 
-        if self.url != c.url:
-            log('Updating channel URL from %s to %s', self.url, c.url, sender=self)
+        if c.status == 302 and hasattr(c, 'headers') and c.headers.get('content-type').startswith('text/html'):
+            log('Warning: Looks like a Wifi authentication page: %s', c.url, sender=self)
+            log('Acting as if the feed was not updated (FIXME!)', sender=self)
+            return (True, None)
+
+        if self.url != c.url and c.status != 302:
+            # The URL has changed, and the status code is not a temporary
+            # redirect, so update the channel's URL accordingly for future use
+            log('Updating channel URL from %s to %s',
+                    self.url, c.url, sender=self)
             self.url = c.url
 
         # update the cover if it's not there
@@ -149,7 +158,7 @@ class podcastChannel(object):
         if hasattr(c.feed, 'title'):
             self.title = c.feed.title
             # Start YouTube-specific title FIX
-            YOUTUBE_PREFIX = 'YouTube :: Videos by'
+            YOUTUBE_PREFIX = 'Videos uploaded by '
             if self.title.startswith(YOUTUBE_PREFIX):
                 self.title = self.title[len(YOUTUBE_PREFIX):] + ' on YouTube'
             # End YouTube-specific title FIX
@@ -206,11 +215,9 @@ class podcastChannel(object):
                         self.count_new -= 1
                         episode = ex
 
-                if not episode.length:
-                    episode.length = resolver.get_real_episode_length(episode)
-
                 episode.save(bulk=True)
 
+        db.commit()
         return ( True, None )
 
     def update_cover(self, force=False):
@@ -239,6 +246,8 @@ class podcastChannel(object):
         self.newest_pubdate_cached = None
         self.update_flag = False # channel is updating or to be updated
         self.iter = None
+        self.foldername = None
+        self.auto_foldername = 1 # automatically generated foldername
 
         # should this channel be synced to devices? (ex: iPod)
         self.sync_to_devices = True
@@ -268,12 +277,6 @@ class podcastChannel(object):
 
     def update_save_dir_size(self):
         self.save_dir_size = util.calculate_size(self.save_dir)
-        
-    def get_filename( self):
-        """Return the MD5 sum of the channel URL"""
-        return md5.new( self.url).hexdigest()
-
-    filename = property(fget=get_filename)
 
     def get_title( self):
         if self.override_title:
@@ -292,6 +295,30 @@ class podcastChannel(object):
     def set_custom_title( self, custom_title):
         custom_title = custom_title.strip()
 
+        # make sure self.foldername is initialized
+        self.get_save_dir()
+
+        # rename folder if custom_title looks sane
+        new_folder_name = self.find_unique_folder_name(custom_title)
+        if len(new_folder_name) > 0 and new_folder_name != self.foldername:
+            log('Changing foldername based on custom title: %s', custom_title, sender=self)
+            new_folder = os.path.join(gl.downloaddir, new_folder_name)
+            old_folder = os.path.join(gl.downloaddir, self.foldername)
+            if os.path.exists(old_folder):
+                if not os.path.exists(new_folder):
+                    # Old folder exists, new folder does not -> simply rename
+                    log('Renaming %s => %s', old_folder, new_folder, sender=self)
+                    os.rename(old_folder, new_folder)
+                else:
+                    # Both folders exist -> move files and delete old folder
+                    log('Moving files from %s to %s', old_folder, new_folder, sender=self)
+                    for file in glob.glob(os.path.join(old_folder, '*')):
+                        shutil.move(file, new_folder)
+                    log('Removing %s', old_folder, sender=self)
+                    shutil.rmtree(old_folder, ignore_errors=True)
+            self.foldername = new_folder_name
+            self.save()
+
         if custom_title != self.__title:
             self.override_title = custom_title
         else:
@@ -318,11 +345,14 @@ class podcastChannel(object):
             f.write('#EXTM3U\n')
 
             for episode in downloaded_episodes:
-                filename = episode.local_filename()
-                if os.path.dirname(filename).startswith(os.path.dirname(m3u_filename)):
-                    filename = filename[len(os.path.dirname(m3u_filename)+os.sep):]
-                f.write('#EXTINF:0,'+self.title+' - '+episode.title+' ('+episode.cute_pubdate()+')\n')
-                f.write(filename+'\n')
+                if episode.was_downloaded(and_exists=True):
+                    filename = episode.local_filename(create=False)
+                    assert filename is not None
+
+                    if os.path.dirname(filename).startswith(os.path.dirname(m3u_filename)):
+                        filename = filename[len(os.path.dirname(m3u_filename)+os.sep):]
+                    f.write('#EXTINF:0,'+self.title+' - '+episode.title+' ('+episode.cute_pubdate()+')\n')
+                    f.write(filename+'\n')
             f.close()
 
     def addDownloadedItem(self, item):
@@ -333,7 +363,9 @@ class podcastChannel(object):
 
             # Update metadata on file (if possible and wanted)
             if gl.config.update_tags and libtagupdate.tagging_supported():
-                filename = item.local_filename()
+                filename = item.local_filename(create=False)
+                assert filename is not None
+
                 try:
                     libtagupdate.update_metadata_on_file(filename, title=item.title, artist=self.title, genre='Podcast')
                 except Exception, e:
@@ -344,21 +376,6 @@ class podcastChannel(object):
     def get_all_episodes(self):
         return db.load_episodes(self, factory = lambda d: podcastItem.create_from_dict(d, self))
 
-    # not used anymore
-    def update_model( self):
-        self.update_save_dir_size()
-        model = self.tree_model
-
-        iter = model.get_iter_first()
-        while iter is not None:
-            self.iter_set_downloading_columns(model, iter)
-            iter = model.iter_next( iter)
-
-    @property
-    def tree_model( self):
-        log('Returning TreeModel for %s', self.url, sender = self)
-        return self.items_liststore()
-
     def iter_set_downloading_columns( self, model, iter, episode=None):
         global ICON_AUDIO_FILE, ICON_VIDEO_FILE
         global ICON_DOWNLOADING, ICON_DELETED, ICON_NEW
@@ -369,7 +386,7 @@ class podcastChannel(object):
         else:
             url = episode.url
 
-        if gl.config.episode_list_descriptions:
+        if gl.config.episode_list_descriptions or gpodder.interface == gpodder.MAEMO:
             icon_size = 32
         else:
             icon_size = 16
@@ -403,7 +420,7 @@ class podcastChannel(object):
 
         model.set( iter, 4, status_icon)
 
-    def items_liststore( self):
+    def get_tree_model(self):
         """
         Return a gtk.ListStore containing episodes for this channel
         """
@@ -411,6 +428,8 @@ class podcastChannel(object):
             gobject.TYPE_BOOLEAN, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, 
             gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
 
+        log('Returning TreeModel for %s', self.url, sender = self)
+        urls = []
         for item in self.get_all_episodes():
             description = item.title_and_description
 
@@ -421,17 +440,74 @@ class podcastChannel(object):
 
             new_iter = new_model.append((item.url, item.title, filelength, 
                 True, None, item.cute_pubdate(), description, util.remove_html_tags(item.description), 
-                item.local_filename(), item.extension()))
+                'XXXXXXXXXXXXXUNUSEDXXXXXXXXXXXXXXXXXXX', item.extension()))
             self.iter_set_downloading_columns( new_model, new_iter, episode=item)
+            urls.append(item.url)
         
         self.update_save_dir_size()
-        return new_model
+        return (new_model, urls)
     
     def find_episode( self, url):
         return db.load_episode(url, factory=lambda x: podcastItem.create_from_dict(x, self))
 
+    @classmethod
+    def find_unique_folder_name(cls, foldername):
+        current_try = util.sanitize_filename(foldername, cls.MAX_FOLDERNAME_LENGTH)
+        next_try_id = 2
+
+        while db.channel_foldername_exists(current_try):
+            current_try = '%s (%d)' % (foldername, next_try_id)
+            next_try_id += 1
+
+        return current_try
+
     def get_save_dir(self):
-        save_dir = os.path.join(gl.downloaddir, self.filename, '')
+        urldigest = hashlib.md5(self.url).hexdigest()
+        sanitizedurl = util.sanitize_filename(self.url, self.MAX_FOLDERNAME_LENGTH)
+        if self.foldername is None or (self.auto_foldername and (self.foldername == urldigest or self.foldername.startswith(sanitizedurl))):
+            # we must change the folder name, because it has not been set manually
+            fn_template = util.sanitize_filename(self.title, self.MAX_FOLDERNAME_LENGTH)
+
+            # if this is an empty string, try the basename
+            if len(fn_template) == 0:
+                log('That is one ugly feed you have here! (Report this to bugs.gpodder.org: %s)', self.url, sender=self)
+                fn_template = util.sanitize_filename(os.path.basename(self.url), self.MAX_FOLDERNAME_LENGTH)
+
+            # If the basename is also empty, use the first 6 md5 hexdigest chars of the URL
+            if len(fn_template) == 0:
+                log('That is one REALLY ugly feed you have here! (Report this to bugs.gpodder.org: %s)', self.url, sender=self)
+                fn_template = urldigest # no need for sanitize_filename here
+
+            # Find a unique folder name for this podcast
+            wanted_foldername = self.find_unique_folder_name(fn_template)
+
+            # if the foldername has not been set, check if the (old) md5 filename exists
+            if self.foldername is None and os.path.exists(os.path.join(gl.downloaddir, urldigest)):
+                log('Found pre-0.15.0 download folder for %s: %s', self.title, urldigest, sender=self)
+                self.foldername = urldigest
+
+            # we have a valid, new folder name in "current_try" -> use that!
+            if self.foldername is not None and wanted_foldername != self.foldername:
+                # there might be an old download folder crawling around - move it!
+                new_folder_name = os.path.join(gl.downloaddir, wanted_foldername)
+                old_folder_name = os.path.join(gl.downloaddir, self.foldername)
+                if os.path.exists(old_folder_name):
+                    if not os.path.exists(new_folder_name):
+                        # Old folder exists, new folder does not -> simply rename
+                        log('Renaming %s => %s', old_folder_name, new_folder_name, sender=self)
+                        os.rename(old_folder_name, new_folder_name)
+                    else:
+                        # Both folders exist -> move files and delete old folder
+                        log('Moving files from %s to %s', old_folder_name, new_folder_name, sender=self)
+                        for file in glob.glob(os.path.join(old_folder_name, '*')):
+                            shutil.move(file, new_folder_name)
+                        log('Removing %s', old_folder_name, sender=self)
+                        shutil.rmtree(old_folder_name, ignore_errors=True)
+            log('Updating foldername of %s to "%s".', self.url, wanted_foldername, sender=self)
+            self.foldername = wanted_foldername
+            self.save()
+
+        save_dir = os.path.join(gl.downloaddir, self.foldername)
 
         # Create save_dir if it does not yet exist
         if not util.make_directory( save_dir):
@@ -460,7 +536,11 @@ class podcastChannel(object):
         episode = db.load_episode(url, lambda c: podcastItem.create_from_dict(c, self))
 
         if episode is not None:
-            util.delete_file(episode.local_filename())
+            filename = episode.local_filename(create=False)
+            if filename is not None:
+                util.delete_file(filename)
+            else:
+                log('Cannot delete episode: %s (I have no filename!)', episode.title, sender=self)
             episode.set_state(db.STATE_DELETED)
 
         self.update_m3u_playlist()
@@ -468,6 +548,7 @@ class podcastChannel(object):
 
 class podcastItem(object):
     """holds data for one object in a channel"""
+    MAX_FILENAME_LENGTH = 200
 
     @staticmethod
     def load(url, channel):
@@ -513,7 +594,14 @@ class podcastItem(object):
                 log('Adding episode with link to file type "%s".', file_type, sender=episode)
                 episode.url = entry.link
 
+        # YouTube specific
+        if not episode.url and hasattr(entry, 'links') and len(entry.links) and hasattr(entry.links[0], 'href'):
+            episode.url = entry.links[0].href
+
         if not episode.url:
+            log('Episode has no URL')
+            log('Episode: %s', episode)
+            log('Entry: %s', entry)
             # This item in the feed has no downloadable enclosure
             return None
 
@@ -552,7 +640,9 @@ class podcastItem(object):
         self.description = ''
         self.link = ''
         self.channel = channel
-        self.pubDate = None
+        self.pubDate = 0
+        self.filename = None
+        self.auto_filename = 1 # automatically generated filename
 
         self.state = db.STATE_NORMAL
         self.is_played = False
@@ -581,6 +671,7 @@ class podcastItem(object):
         self.is_played = False
         if save:
             self.save()
+            db.commit()
 
     @staticmethod
     def create_from_dict(d, channel):
@@ -597,13 +688,13 @@ class podcastItem(object):
         disables the description when the config variable
         "episode_list_descriptions" is not set.
         """
-        if gl.config.episode_list_descriptions:
+        if gl.config.episode_list_descriptions and gpodder.interface != gpodder.MAEMO:
             return '%s\n<small>%s</small>' % (saxutils.escape(self.title), saxutils.escape(self.one_line_description()))
         else:
             return saxutils.escape(self.title)
 
     def age_in_days(self):
-        return util.file_age_in_days(self.local_filename())
+        return util.file_age_in_days(self.local_filename(create=False))
 
     def is_old(self):
         return self.age_in_days() > gl.config.episode_old_age
@@ -618,7 +709,7 @@ class podcastItem(object):
         if not lines or lines[0] == '':
             return _('No description available')
         else:
-            return ' '.join((l.strip() for l in lines if l.strip() != ''))
+            return ' '.join(lines)
 
     def delete_from_disk(self):
         try:
@@ -626,26 +717,127 @@ class podcastItem(object):
         except:
             log('Cannot delete episode from disk: %s', self.title, traceback=True, sender=self)
 
-    def local_filename( self):
+    @classmethod
+    def find_unique_file_name(cls, url, filename, extension):
+        current_try = util.sanitize_filename(filename, cls.MAX_FILENAME_LENGTH)+extension
+        next_try_id = 2
+        lookup_url = None
+
+        while db.episode_filename_exists(current_try):
+            if next_try_id == 2:
+                # If we arrive here, current_try has a collision, so
+                # try to resolve the URL for a better basename
+                log('Filename collision: %s - trying to resolve...', current_try)
+                url = util.get_real_url(url)
+                (episode_filename, extension_UNUSED) = util.filename_from_url(url)
+                current_try = util.sanitize_filename(episode_filename, cls.MAX_FILENAME_LENGTH)+extension
+                if not db.episode_filename_exists(current_try):
+                    log('Filename %s is available - collision resolved.', current_try)
+                    return current_try
+                else:
+                    log('Continuing search with %s as basename...', current_try)
+
+            current_try = '%s (%d)%s' % (filename, next_try_id, extension)
+            next_try_id += 1
+
+        return current_try
+
+    def local_filename(self, create, force_update=False, check_only=False):
+        """Get (and possibly generate) the local saving filename
+
+        Pass create=True if you want this function to generate a
+        new filename if none exists. You only want to do this when
+        planning to create/download the file after calling this function.
+
+        Normally, you should pass create=False. This will only
+        create a filename when the file already exists from a previous
+        version of gPodder (where we used md5 filenames). If the file
+        does not exist (and the filename also does not exist), this
+        function will return None.
+
+        If you pass force_update=True to this function, it will try to
+        find a new (better) filename and move the current file if this
+        is the case. This is useful if (during the download) you get
+        more information about the file, e.g. the mimetype and you want
+        to include this information in the file name generation process.
+
+        If check_only=True is passed to this function, it will never try
+        to rename the file, even if would be a good idea. Use this if you
+        only want to check if a file exists.
+
+        The generated filename is stored in the database for future access.
+        """
         ext = self.extension()
 
-        # For compatibility with already-downloaded episodes,
-        # we accept md5 filenames if they are downloaded now.
-        md5_filename = os.path.join(self.channel.save_dir, md5.new(self.url).hexdigest()+ext)
-        if os.path.exists(md5_filename) or not gl.config.experimental_file_naming:
-            return md5_filename
+        # For compatibility with already-downloaded episodes, we
+        # have to know md5 filenames if they are downloaded already
+        urldigest = hashlib.md5(self.url).hexdigest()
 
-        # If the md5 filename does not exist, 
-        ( episode, e ) = util.filename_from_url(self.url)
-        episode = util.sanitize_filename(episode) + ext
+        if not create and self.filename is None:
+            urldigest_filename = os.path.join(self.channel.save_dir, urldigest+ext)
+            if os.path.exists(urldigest_filename):
+                # The file exists, so set it up in our database
+                log('Recovering pre-0.15.0 file: %s', urldigest_filename, sender=self)
+                self.filename = urldigest+ext
+                self.auto_filename = 1
+                self.save()
+                return urldigest_filename
+            return None
 
-        # If the episode filename looks suspicious,
-        # we still return the md5 filename to be on
-        # the safe side of the fence ;)
-        if len(episode) == 0 or episode.startswith('redirect.'):
-            return md5_filename
-        filename = os.path.join(self.channel.save_dir, episode)
-        return filename
+        # We only want to check if the file exists, so don't try to
+        # rename the file, even if it would be reasonable. See also:
+        # http://bugs.gpodder.org/attachment.cgi?id=236
+        if check_only:
+            if self.filename is None:
+                return None
+            else:
+                return os.path.join(self.channel.save_dir, self.filename)
+
+        if self.filename is None or force_update or (self.auto_filename and self.filename == urldigest+ext):
+            # Try to find a new filename for the current file
+            (episode_filename, extension_UNUSED) = util.filename_from_url(self.url)
+            fn_template = util.sanitize_filename(episode_filename, self.MAX_FILENAME_LENGTH)
+
+            if 'redirect' in fn_template:
+                # This looks like a redirection URL - force URL resolving!
+                log('Looks like a redirection to me: %s', self.url, sender=self)
+                url = util.get_real_url(self.url)
+                log('Redirection resolved to: %s', url, sender=self)
+                (episode_filename, extension_UNUSED) = util.filename_from_url(url)
+                fn_template = util.sanitize_filename(episode_filename, self.MAX_FILENAME_LENGTH)
+
+            # If the basename is empty, use the md5 hexdigest of the URL
+            if len(fn_template) == 0 or fn_template.startswith('redirect.'):
+                log('Report to bugs.gpodder.org: Podcast at %s with episode URL: %s', self.channel.url, self.url, sender=self)
+                fn_template = urldigest
+
+            # Find a unique filename for this episode
+            wanted_filename = self.find_unique_file_name(self.url, fn_template, ext)
+
+            # We populate the filename field the first time - does the old file still exist?
+            if self.filename is None and os.path.exists(os.path.join(self.channel.save_dir, urldigest+ext)):
+                log('Found pre-0.15.0 downloaded file: %s', urldigest, sender=self)
+                self.filename = urldigest+ext
+
+            # The old file exists, but we have decided to want a different filename
+            if self.filename is not None and wanted_filename != self.filename:
+                # there might be an old download folder crawling around - move it!
+                new_file_name = os.path.join(self.channel.save_dir, wanted_filename)
+                old_file_name = os.path.join(self.channel.save_dir, self.filename)
+                if os.path.exists(old_file_name) and not os.path.exists(new_file_name):
+                    log('Renaming %s => %s', old_file_name, new_file_name, sender=self)
+                    os.rename(old_file_name, new_file_name)
+                elif force_update and not os.path.exists(old_file_name):
+                    # When we call force_update, the file might not yet exist when we
+                    # call it from the downloading code before saving the file
+                    log('Choosing new filename: %s', new_file_name, sender=self)
+                else:
+                    log('Warning: %s exists or %s does not.', new_file_name, old_file_name, sender=self)
+            log('Updating filename of %s to "%s".', self.url, wanted_filename, sender=self)
+            self.filename = wanted_filename
+            self.save()
+
+        return os.path.join(self.channel.save_dir, self.filename)
 
     def extension( self):
          ( filename, ext ) = util.filename_from_url(self.url)
@@ -665,7 +857,11 @@ class podcastItem(object):
         db.mark_episode(self.url, is_played=True)
 
     def file_exists(self):
-        return os.path.exists(self.local_filename())
+        filename = self.local_filename(create=False, check_only=True)
+        if filename is None:
+            return False
+        else:
+            return os.path.exists(filename)
 
     def was_downloaded(self, and_exists=False):
         if self.state != db.STATE_DOWNLOADED:
@@ -692,11 +888,25 @@ class podcastItem(object):
     
     @property
     def published( self):
+        """
+        Returns published date as YYYYMMDD (or 00000000 if not available)
+        """
         try:
             return datetime.datetime.fromtimestamp(self.pubDate).strftime('%Y%m%d')
         except:
             log( 'Cannot format pubDate for "%s".', self.title, sender = self)
             return '00000000'
+
+    @property
+    def pubtime(self):
+        """
+        Returns published time as HHMM (or 0000 if not available)
+        """
+        try:
+            return datetime.datetime.fromtimestamp(self.pubDate).strftime('%H%M')
+        except:
+            log('Cannot format pubDate (time) for "%s".', self.title, sender=self)
+            return '0000'
     
     def cute_pubdate(self):
         result = util.format_date(self.pubDate)
@@ -708,8 +918,11 @@ class podcastItem(object):
     pubdate_prop = property(fget=cute_pubdate)
 
     def calculate_filesize( self):
+        filename = self.local_filename(create=False)
+        if filename is None:
+            log('calculate_filesized called, but filename is None!', sender=self)
         try:
-            self.length = os.path.getsize(self.local_filename())
+            self.length = os.path.getsize(filename)
         except:
             log( 'Could not get filesize for %s.', self.url)
 
@@ -734,16 +947,17 @@ class podcastItem(object):
 
 
 def update_channel_model_by_iter( model, iter, channel, color_dict,
-    cover_cache=None, max_width=0, max_height=0 ):
+        cover_cache=None, max_width=0, max_height=0, initialize_all=False):
 
     count_downloaded = channel.stat(state=db.STATE_DOWNLOADED)
     count_new = channel.stat(state=db.STATE_NORMAL, is_played=False)
     count_unplayed = channel.stat(state=db.STATE_DOWNLOADED, is_played=False)
 
     channel.iter = iter
-    model.set(iter, 0, channel.url)
-    model.set(iter, 1, channel.title)
+    if initialize_all:
+        model.set(iter, 0, channel.url)
 
+    model.set(iter, 1, channel.title)
     title_markup = saxutils.escape(channel.title)
     description_markup = saxutils.escape(util.get_first_line(channel.description) or _('No description available'))
     d = []
@@ -773,23 +987,26 @@ def update_channel_model_by_iter( model, iter, channel, color_dict,
     else:
         model.set(iter, 7, False)
 
-    # Load the cover if we have it, but don't download
-    # it if it's not available (to avoid blocking here)
-    pixbuf = services.cover_downloader.get_cover(channel, avoid_downloading=True)
-    new_pixbuf = None
-    if pixbuf is not None:
-        new_pixbuf = util.resize_pixbuf_keep_ratio(pixbuf, max_width, max_height, channel.url, cover_cache)
-    model.set(iter, 5, new_pixbuf or pixbuf)
+    if initialize_all:
+        # Load the cover if we have it, but don't download
+        # it if it's not available (to avoid blocking here)
+        pixbuf = services.cover_downloader.get_cover(channel, avoid_downloading=True)
+        new_pixbuf = None
+        if pixbuf is not None:
+            new_pixbuf = util.resize_pixbuf_keep_ratio(pixbuf, max_width, max_height, channel.url, cover_cache)
+        model.set(iter, 5, new_pixbuf or pixbuf)
 
 def channels_to_model(channels, color_dict, cover_cache=None, max_width=0, max_height=0):
     new_model = gtk.ListStore( str, str, str, gtk.gdk.Pixbuf, int,
         gtk.gdk.Pixbuf, str, bool, str )
 
+    urls = []
     for channel in channels:
-        update_channel_model_by_iter( new_model, new_model.append(), channel,
-            color_dict, cover_cache, max_width, max_height )
+        update_channel_model_by_iter(new_model, new_model.append(), channel,
+            color_dict, cover_cache, max_width, max_height, True)
+        urls.append(channel.url)
 
-    return new_model
+    return (new_model, urls)
 
 
 def load_channels():
