@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -169,7 +169,11 @@ class Device(services.ObservableService):
 
     def close(self):
         self.notify('status', _('Writing data to disk'))
-        successful_sync = not os.system('sync')
+        if gl.config.sync_disks_after_transfer:
+            successful_sync = (os.system('sync') == 0)
+        else:
+            log('Not syncing disks. Unmount your device before unplugging.', sender=self)
+            successful_sync = True
         self.notify('done')
         self.notify('post-done', self, successful_sync)
         return True
@@ -201,6 +205,32 @@ class Device(services.ObservableService):
                 track.delete_from_disk()
         return True
 
+    def convert_track(self, episode):
+        filename = episode.local_filename(create=False)
+        # The file has to exist, if we ought to transfer it, and therefore,
+        # local_filename(create=False) must never return None as filename
+        assert filename is not None
+        (fn, extension) = os.path.splitext(filename)
+        if libconverter.converters.has_converter(extension):
+            if gl.config.disable_pre_sync_conversion:
+                log('Pre-sync conversion is not enabled, set disable_pre_sync_conversion to "False" to enable')
+                return filename
+
+            log('Converting: %s', filename, sender=self)
+            callback_status = lambda percentage: self.notify('sub-progress', int(percentage))
+            local_filename = libconverter.converters.convert(filename, callback=callback_status)
+
+            if local_filename is None:
+                log('Cannot convert %s', original_filename, sender=self)
+                return filename
+
+            if not libtagupdate.update_metadata_on_file(local_filename, title=episode.title, artist=episode.channel.title):
+                log('Could not set metadata on converted file %s', local_filename, sender=self)
+
+            return str(local_filename)
+
+        return filename
+
     def remove_tracks(self, tracklist=[]):
         for id, track in enumerate(tracklist):
             if self.cancelled:
@@ -345,7 +375,10 @@ class iPodDevice(Device):
                 self.set_podcast_flags(track)
                 return True
 
-        original_filename = str(episode.local_filename())
+        original_filename = episode.local_filename(create=False)
+        # The file has to exist, if we ought to transfer it, and therefore,
+        # local_filename(create=False) must never return None as filename
+        assert original_filename is not None
         local_filename = original_filename
 
         if util.calculate_size(original_filename) > self.get_free_space():
@@ -354,20 +387,7 @@ class iPodDevice(Device):
             self.cancelled = True
             return False
 
-        (fn, extension) = os.path.splitext(original_filename)
-        if libconverter.converters.has_converter(extension):
-            log('Converting: %s', original_filename, sender=self)
-            callback_status = lambda percentage: self.notify('sub-progress', int(percentage))
-            local_filename = libconverter.converters.convert(original_filename, callback=callback_status)
-
-            if not libtagupdate.update_metadata_on_file(local_filename, title=episode.title, artist=episode.channel.title):
-                log('Could not set metadata on converted file %s', local_filename, sender=self)
-
-            if local_filename is None:
-                log('Cannot convert %s', original_filename, sender=self)
-                return False
-            else:
-                local_filename = str(local_filename)
+        local_filename = self.convert_track(episode)
 
         (fn, extension) = os.path.splitext(local_filename)
         if extension.lower().endswith('ogg'):
@@ -411,7 +431,7 @@ class iPodDevice(Device):
 
         gpod.itdb_track_add(self.itdb, track, -1)
         gpod.itdb_playlist_add_track(self.podcasts_playlist, track, -1)
-        gpod.itdb_cp_track_to_ipod( track, local_filename, None)
+        gpod.itdb_cp_track_to_ipod(track, str(local_filename), None)
 
         # If the file has been converted, delete the temporary file here
         if local_filename != original_filename:
@@ -522,7 +542,7 @@ class MP3PlayerDevice(Device):
         else:
             folder = self.destination
 
-        from_file = util.sanitize_encoding(episode.local_filename())
+        from_file = util.sanitize_encoding(self.convert_track(episode))
         filename_base = util.sanitize_filename(episode.sync_filename(), self.MAX_FILENAME_LENGTH)
 
         to_file = filename_base + os.path.splitext(from_file)[1].lower()
@@ -848,11 +868,12 @@ class MTPDevice(Device):
 
     def add_track(self, episode):
         self.notify('status', _('Adding %s...') % episode.title)
-        log("sending " + str(episode.local_filename()) + " (" + episode.title + ").", sender=self)
+        filename = self.convert_track(episode)
+        log("sending " + filename + " (" + episode.title + ").", sender=self)
 
         try:
             # verify free space
-            needed = util.calculate_size(episode.local_filename())
+            needed = util.calculate_size(filename)
             free = self.get_free_space()
             if needed > free:
                 log('Not enough space on device %s: %s available, but need at least %s', self.get_name(), util.format_filesize(free), util.format_filesize(needed), sender=self)
@@ -866,10 +887,10 @@ class MTPDevice(Device):
             metadata.album = str(episode.channel.title)
             metadata.genre = "podcast"
             metadata.date = self.__date_to_mtp(episode.pubDate)
-            metadata.duration = get_track_length(str(episode.local_filename()))
+            metadata.duration = get_track_length(str(filename))
 
             # send the file
-            self.__MTPDevice.send_track_from_file( str(episode.local_filename()), episode.basename, metadata, 0, callback=self.__callback)
+            self.__MTPDevice.send_track_from_file( filename, episode.basename, metadata, 0, callback=self.__callback)
         except:
             log('unable to add episode %s', episode.title, sender=self, traceback=True)
             return False
