@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -103,7 +103,7 @@ def make_directory( path):
     return True
 
 
-def normalize_feed_url( url):
+def normalize_feed_url(url):
     """
     Converts any URL to http:// or ftp:// so that it can be 
     used with "wget". If the URL cannot be converted (invalid
@@ -117,26 +117,37 @@ def normalize_feed_url( url):
     simply assume the user intends to add a http:// feed.
     """
 
-    if not url or len( url) < 8:
+    if not url or len(url) < 8:
         return None
 
+    # Assume HTTP for URLs without scheme
     if not '://' in url:
         url = 'http://' + url
 
-    if url.startswith('itms://'):
+    # The scheme of the URL should be all-lowercase
+    (scheme, rest) = url.split('://', 1)
+    scheme = scheme.lower()
+
+    # Remember to parse iTunes XML for itms:// URLs
+    do_parse_itunes_xml = (scheme == 'itms')
+
+    # feed://, itpc:// and itms:// are really http://
+    if scheme in ('feed', 'itpc', 'itms'):
+        scheme = 'http'
+
+    # Re-assemble our URL
+    url = scheme + '://' + rest
+
+    # If we had an itms:// URL, parse XML
+    if do_parse_itunes_xml:
         url = parse_itunes_xml(url)
 
     # Links to "phobos.apple.com"
     url = itunes_discover_rss(url)
-    if url is None:
-        return None
     
-    if url.startswith( 'http://') or url.startswith( 'https://') or url.startswith( 'ftp://'):
+    if scheme in ('http', 'https', 'ftp'):
         return url
 
-    if url.startswith('feed://') or url.startswith('itpc://'):
-        return 'http://' + url[7:]
-
     return None
 
 
@@ -245,7 +256,7 @@ def file_age_to_string(days):
     ''
     >>> file_age_to_string(1)
     'one day ago'
-    >>> file_age_to_String(2)
+    >>> file_age_to_string(2)
     '2 days ago'
     """
     if days == 1:
@@ -290,7 +301,11 @@ def format_date(timestamp):
 
     today = time.localtime()[:3]
     yesterday = time.localtime(time.time() - seconds_in_a_day)[:3]
-    timestamp_date = time.localtime(timestamp)[:3]
+    try:
+        timestamp_date = time.localtime(timestamp)[:3]
+    except ValueError, ve:
+        log('Warning: Cannot convert timestamp', sender=self, traceback=True)
+        return None
     
     if timestamp_date == today:
        return _('Today')
@@ -412,6 +427,40 @@ def extension_from_mimetype(mimetype):
     """
     return mimetypes.guess_extension(mimetype) or ''
 
+
+def extension_correct_for_mimetype(extension, mimetype):
+    """
+    Check if the given filename extension (e.g. ".ogg") is a possible
+    extension for a given mimetype (e.g. "application/ogg") and return
+    a boolean value (True if it's possible, False if not). Also do
+
+    >>> extension_correct_for_mimetype('.ogg', 'application/ogg')
+    True
+    >>> extension_correct_for_mimetype('.ogv', 'video/ogg')
+    True
+    >>> extension_correct_for_mimetype('.ogg', 'audio/mpeg')
+    False
+    >>> extension_correct_for_mimetype('mp3', 'audio/mpeg')
+    Traceback (most recent call last):
+      ...
+    ValueError: "mp3" is not an extension (missing .)
+    >>> extension_correct_for_mimetype('.mp3', 'audio mpeg')
+    Traceback (most recent call last):
+      ...
+    ValueError: "audio mpeg" is not a mimetype (missing /)
+    """
+    if not '/' in mimetype:
+        raise ValueError('"%s" is not a mimetype (missing /)' % mimetype)
+    if not extension.startswith('.'):
+        raise ValueError('"%s" is not an extension (missing .)' % extension)
+
+    # Create a "default" extension from the mimetype, e.g. "application/ogg"
+    # becomes ".ogg", "audio/mpeg" becomes ".mpeg", etc...
+    default = ['.'+mimetype.split('/')[-1]]
+
+    return extension in default+mimetypes.guess_all_extensions(mimetype)
+
+
 def filename_from_url(url):
     """
     Extracts the filename and (lowercase) extension (with dot)
@@ -457,7 +506,7 @@ def file_type_by_extension( extension):
     None if the file type cannot be determined.
     """
     types = {
-            'audio': [ 'mp3', 'ogg', 'wav', 'wma', 'aac', 'm4a' ],
+            'audio': [ 'mp3', 'ogg', 'wav', 'wma', 'aac', 'm4a', 'm4b' ],
             'video': [ 'mp4', 'avi', 'mpg', 'mpeg', 'm4v', 'mov', 'divx', 'flv', 'wmv', '3gp' ],
     }
 
@@ -606,11 +655,18 @@ def format_desktop_command( command, filename):
 
     for key, value in items.items():
         if command.find( key) >= 0:
-            return command.replace( key, value)
+            return command.replace( key, '"%s"' % value)
 
     return '%s "%s"' % ( command, filename )
 
 
+def get_real_url(url):
+    """
+    Gets the real URL of a file and resolves all redirects.
+    """
+    return urllib.urlopen(url).geturl()
+
+
 def find_command( command):
     """
     Searches the system's PATH for a specific command that is
@@ -821,7 +877,7 @@ def format_seconds_to_hour_min_sec(seconds):
 
     >>> format_seconds_to_hour_min_sec(3834)
     '1 hour, 3 minutes and 54 seconds'
-    >>> format_seconds_to_hour_min_sec(2600)
+    >>> format_seconds_to_hour_min_sec(3600)
     '1 hour'
     >>> format_seconds_to_hour_min_sec(62)
     '1 minute and 2 seconds'
@@ -1026,3 +1082,57 @@ def resize_pixbuf_keep_ratio(pixbuf, max_width, max_height, key=None, cache=None
 
     return result
 
+# matches http:// and ftp:// and mailto://
+protocolPattern = re.compile(r'^\w+://')
+
+def isabs(string):
+    """
+    @return true if string is an absolute path or protocoladdress
+    for addresses beginning in http:// or ftp:// or ldap:// -
+    they are considered "absolute" paths.
+    Source: http://code.activestate.com/recipes/208993/
+    """
+    if protocolPattern.match(string): return 1
+    return os.path.isabs(string)
+
+def rel2abs(path, base = os.curdir):
+    """ converts a relative path to an absolute path.
+
+    @param path the path to convert - if already absolute, is returned
+    without conversion.
+    @param base - optional. Defaults to the current directory.
+    The base is intelligently concatenated to the given relative path.
+    @return the relative path of path from base
+    Source: http://code.activestate.com/recipes/208993/
+    """
+    if isabs(path): return path
+    retval = os.path.join(base,path)
+    return os.path.abspath(retval)
+
+def commonpath(l1, l2, common=[]):
+    """
+    helper functions for relpath
+    Source: http://code.activestate.com/recipes/208993/
+    """
+    if len(l1) < 1: return (common, l1, l2)
+    if len(l2) < 1: return (common, l1, l2)
+    if l1[0] != l2[0]: return (common, l1, l2)
+    return commonpath(l1[1:], l2[1:], common+[l1[0]])
+
+def relpath(p1, p2):
+    """
+    Finds relative path from p1 to p2
+    Source: http://code.activestate.com/recipes/208993/
+    """
+    pathsplit = lambda s: s.split(os.path.sep)
+
+    (common,l1,l2) = commonpath(pathsplit(p1), pathsplit(p2))
+    p = []
+    if len(l1) > 0:
+        p = [ ('..'+os.sep) * len(l1) ]
+    p = p + l2
+    if len(p) is 0:
+        return "."
+
+    return os.path.join(*p)
+
